/*-
 * Copyright 1979, 1980, 1983, 1986, 1988, 1989, 1991, 1992, 1993, 1994
 * 	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by the University of
 * California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

#ifndef _MACHINE_ASI_H_
#define _MACHINE_ASI_H_ 1

#define ASI_AFAR			0x4d	/* Async fault addr reg */
#define ASI_AFSR			0x4c	/* Async fault status reg */

#define ASI_NUCLEUS				0x04
#define ASI_NUCLEUS_LITTLE			0x0c
#define ASI_NUCLEUS_QUAD_LDD			0x24
#define ASI_NUCLEUS_QUAD_LDD_LITTLE		0x2c
#define ASI_AS_IF_USER_PRIMARY			0x10
#define ASI_AS_IF_USER_PRIMARY_LITTLE		0x18
#define ASI_AS_IF_USER_SECONDARY		0x11
#define ASI_AS_IF_USER_SECONDARY_LITTLE		0x19
#define ASI_PRIMARY				0x80
#define ASI_PRIMARY_LITTLE			0x88
#define ASI_PRIMARY_NO_FAULT			0x82
#define ASI_PRIMARY_NO_FAULT_LITTLE		0x8a
#define ASI_PST16_PRIMARY			0xc2
#define ASI_PST16_PRIMARY_LITTLE		0xca
#define ASI_PST16_SECONDARY			0xc3
#define ASI_PST16_SECONDARY_LITTLE		0xcb
#define ASI_PST32_PRIMARY			0xc4
#define ASI_PST32_PRIMARY_LITTLE		0xcc
#define ASI_PST32_SECONDARY			0xc5
#define ASI_PST32_SECONDARY_LITTLE		0xcd
#define ASI_PST8_PRIMARY			0xc0
#define ASI_PST8_PRIMARY_LITTLE			0xc8
#define ASI_PST8_SECONDARY			0xc1
#define ASI_PST8_SECONDARY_LITTLE		0xc9
#define ASI_SECONDARY				0x81
#define ASI_SECONDARY_LITTLE			0x89
#define ASI_SECONDARY_NO_FAULT			0x83
#define ASI_SECONDARY_NO_FAULT_LITTLE		0x8b
#define ASI_PHYS_USE_EC				0x14
#define ASI_PHYS_USE_EC_LITTLE			0x1c
#define ASI_PHYS_BYPASS_EC_WITH_EBIT		0x15
#define ASI_PHYS_BYPASS_EC_WITH_EBIT_LITTLE	0x1d

#define ASI_BLOCK_AS_IF_USER_PRIMARY		0x70
#define ASI_BLOCK_AS_IF_USER_PRIMARY_LITTLE	0x78
#define ASI_BLK_AS_IF_USER_SECONDARY		0x71
#define ASI_BLK_AS_IF_USER_SECONDARY_LITTLE	0x79
#define ASI_BLK_COMMIT_PRIMARY			0xe0
#define ASI_BLK_COMMIT_SECONDARY		0xe1
#define ASI_BLOCK_PRIMARY			0xf0
#define ASI_BLOCK_PRIMARY_LITTLE		0xf8
#define ASI_BLOCK_SECONDARY			0xf1
#define ASI_BLOCK_SECONDARY_LITTLE		0xf9

#define ASI_FL16_PRIMARY		0xd2
#define ASI_FL16_PRIMARY_LITTLE		0xda
#define ASI_FL16_SECONDARY		0xd3
#define ASI_FL16_SECONDARY_LITTLE	0xdb
#define ASI_FL8_PRIMARY			0xd0
#define ASI_FL8_PRIMARY_LITTLE		0xd8
#define ASI_FL8_SECONDARY		0xd1
#define ASI_FL8_SECONDARY_LITTLE	0xd9

#define ASI_DMMU			0x58
#define ASI_DMMU_DEMAP			0x5f
#define ASI_DMMU_TSB_64KB_PTR_REG	0x5a
#define ASI_DMMU_TSB_8KB_PTR_REG	0x59
#define ASI_DMMU_TSB_DIRECT_PTR_REG	0x5b
#define ASI_DTLB_DATA_ACCESS_REG	0x5d
#define ASI_DTLB_DATA_IN_REG		0x5c
#define ASI_DTLB_TAG_READ_REG		0x5e
#define ASI_IMMU			0x50
#define ASI_IMMU_DEMAP			0x57
#define ASI_IMMU_TSB_64KB_PTR_REG	0x52
#define ASI_IMMU_TSB_8KB_PTR_REG	0x51
#define ASI_ITLB_DATA_ACCESS_REG	0x55
#define ASI_ITLB_DATA_IN_REG		0x54
#define ASI_ITLB_TAG_READ_REG		0x56

#define ASI_INTR_DISPATCH_STATUS	0x48
#define ASI_INTR_RECEIVE		0x49

#define ASI_DCACHE_DATA			0x46
#define ASI_DCACHE_TAG			0x47
#define ASI_ICACHE_INSTR		0x66
#define ASI_ICACHE_NEXT_FIELD		0x6f
#define ASI_ICACHE_PRE_DECODE		0x6e
#define ASI_ICACHE_TAG			0x67

#define ASI_ECACHE_R			0x7e
#define ASI_ECACHE_TAG_DATA		0x4e
#define ASI_ECACHE_W			0x76
#define ASI_ESTATE_ERROR_EN_REG		0x4b

#define LSU_CONTROL_REG			0x45

#define ASI_UDBH_CONTROL_REG_READ	0x7f
#define ASI_UDBH_CONTROL_REG_WRITE	0x77
#define ASI_UDBH_ERROR_REG_READ		0x7f
#define ASI_UDBH_ERROR_REG_WRITE	0x77
#define ASI_UDBL_CONTROL_REG_READ	0x7f
#define ASI_UDBL_CONTROL_REG_WRITE	0x77
#define ASI_UDBL_ERROR_REG_READ		0x7f
#define ASI_UDBL_ERROR_REG_WRITE	0x77
#define ASI_UDB_INTR_R			0x7f /* Incoming interrupt vector data
						registers 0, 1, 2 */
#define ASI_UDB_INTR_W			0x77 /* Outgoing interrupt vector data
						registers 0, 1, 2 and vector
						dispatch */
#define ASI_UPA_CONFIG_REG		0x4a /* UPA configuration register */

#endif /* !_MACHIINE_ASM_H_ */
