/*-
 * Copyright 1979, 1980, 1983, 1986, 1988, 1989, 1991, 1992, 1993, 1994
 * 	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by the University of
 * California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

#ifndef _MACHINE_CPU_H_
#define _MACHINE_CPU_H_ 1

/*
 * sparc v9 pstate definitions.
 */
#define PSTATE_AG	0x1		/* alternate globals */
#define PSTATE_IE	0x2		/* interrupt enable */
#define PSTATE_PRIV	0x4		/* privileged mode */
#define PSTATE_AM	0x8		/* 32 bit address masking enable */
#define PSTATE_PEF	0x10		/* FPU enable */
#define PSTATE_RED	0x20		/* RED state */
#define PSTATE_TSO	0x0		/* total store order */
#define PSTATE_PSO	0x40		/* partial store order */
#define PSTATE_RMO	0x80		/* relaxed memory order */
#define PSTATE_TLE	0x100		/* trap little endian */ 
#define PSTATE_CLE	0x200		/* current little endian */
#define PSTATE_MG	0x400		/* MMU globals enable */
#define PSTATE_IG	0x800		/* interrupt globals enable */

#define PSTATE_KERN	(PSTATE_TSO | PSTATE_PEF | PSTATE_IE | PSTATE_PRIV)

/* v9 Version Register */
#define VER_MANUF(ver) ((ver) >> 48)
#define VER_MAXTL(ver) (((ver) >> 8) & 0xff)
#define VER_MAXWIN(ver) ((ver) & 0xf)
#define VER_IMPL(ver) (((ver) >> 32) & 0xffff)
#define VER_MASK(ver) (((ver) >> 24) & 0xff)

/*
 * XXX Add TSTATE, FPRS registers.
 */
#define CPU_CONSDEV	1

#if !defined(__ASSEMBLER__)

extern void ident_cpu();

#define gettba() ({unsigned long x; __asm__ __volatile__("rdpr %%tba, %0": "=r"(x)); x;})
#define getpstate() ({unsigned long x; __asm__ __volatile__("rdpr %%pstate, %0": "=r"(x)); x;})
#define getpil() ({unsigned long x; __asm__ __volatile__("rdpr %%pil, %0": "=r"(x)); x;})
#define gettick() ({unsigned long x; __asm__ __volatile__("rdpr %%tick, %0": "=r"(x)); x;})
#define gettick_cmpr() ({unsigned long x; __asm__ __volatile__("rd %%tick_cmpr, %0": "=r"(x)); x;})
#define settick_cmpr(t) {unsigned long x; __asm__ __volatile__("rd %%tick, %0; add %0, %1, %0; wr %0, 0, %%tick_cmpr" : "=&r"(x) : "r"(t), "r"(1UL<<63));}

#endif

#endif /* !_MACHIINE_CPU_H_ */
