/*-
 * Copyright 1979, 1980, 1983, 1986, 1988, 1989, 1991, 1992, 1993, 1994
 * 	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by the University of
 * California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

#ifndef _MACHINE_PMAP_H_
#define _MACHINE_PMAP_H_ 1

/*
 * sparc v9 mmu definitons.
 */

/* TAG portion of the TTE */
#define TTE_VAMASK	0x000003ffffffffff	/* VA_tag<63:22> */
#define TTE_VASHIFT	22
#define TTE_CONTEXT	0x1fff000000000000	/* context */ 
#define TTE_GLOBAL1	0x8000000000000000	/* global bit */

/* Data portion of the TTE and i/d-TLB Data In/Access Registers */
#define TTE_VALID	0x8000000000000000	/* valid bit */
#define TTE_SZ8K	0x0000000000000000	/* 8 Kb page */
#define TTE_SZ64K	0x2000000000000000	/* 64 Kb page */
#define TTE_SZ512K	0x4000000000000000	/* 64 Kb page */
#define TTE_SZ4M	0x6000000000000000	/* 4 Mb page */
#define TTE_NFO		0x1000000000000000	/* non-faulting loads only */
#define TTE_IE		0x0800000000000000	/* invert endianness */
#define TTE_PAMASK	0x000001ffffffe000	/* PA<40:13> */
#define TTE_LOCK	0x0000000000000040	/* lock bit */ 
#define TTE_CP		0x0000000000000020	/* cacheable-in-physically-indexed-cache */
#define TTE_CV		0x0000000000000010	/* cacheable-in-virtually-indexed-cache */
#define TTE_E		0x0000000000000008	/* side effect bit */
#define TTE_P		0x0000000000000004	/* privileged access only */
#define TTE_W		0x0000000000000002	/* writeable bit */
#define TTE_GLOBAL2	0x0000000000000001	/* global bit */

#define TTE_KERN (TTE_VALID | TTE_CP | TTE_CV | TTE_P | TTE_W | TTE_GLOBAL2)

/* Tag Access Register */
#define TAG_ACCESS_REG	0x30
#define TAG_VAMASK	0xffffffffffffe000
#define TAG_CONTEXT	0x0000000000001fff

/* i/d-TLB Demap Operation */
#define DEMAP_TYPE_CONTEXT	0x0000000000000040
#define DEMAP_CONTEXT		0x0000000000000030

#define TSB_REG		0x28

#define PRIMARY_CONTEXT		0x08
#define SECONDARY_CONTEXT 	0x10

#define PT_SHIFT	13
#define PD2_SHIFT	(PT_SHIFT+10)
#define PD1_SHIFT	(PD2_SHIFT+10)
#define VA_SHIFT	(PD1_SHIFT+10)

#define PT_SIZE		PAGE_SIZE
#define PT_MASK		PAGE_MASK
#define PD2_SIZE	(1UL<<PD2_SHIFT)
#define PD2_MASK	(PD2_SIZE-1)
#define PD1_SIZE	(1UL<<PD1_SHIFT)
#define PD1_MASK	(PD1_SIZE-1)
#define VA_SIZE		(1UL<<VA_SHIFT)
#define VA_MASK		(VA_SIZE-1)

#define NUM_PD2(x)	(((x) + (PD2_SIZE - 1))>>PD2_SHIFT)
#define PT_OFS(v)	(((v) & PD2_MASK)>>PT_SHIFT)
#define PD2_OFS(v)	(((v) & PD1_MASK)>>PD2_SHIFT)
#define PD1_OFS(v)	(((v) & VA_MASK)>>PD1_SHIFT)

#define VADDR(x, y, z)	(((x)<<PD1_SHIFT)|((y)<<PD2_SHIFT)|((z)<<PT_SHIFT))

#if !defined (__ASSEMBLER__)

#define get_primary_context() ({unsigned long x; __asm__ __volatile__("ldxa [%1] %2, %0": "=&r"(x):"r"(PRIMARY_CONTEXT),"i"(ASI_DMMU)); x;}) 

#define get_secondary_context() ({unsigned long x; __asm__ __volatile__("ldxa [%1] %2, %0": "=r"(x):"r"(SECONDARY_CONTEXT),"i"(ASI_DMMU)); x;}) 

#define set_primary_context(c) ({unsigned long pc, x = PRIMARY_CONTEXT; __asm__ __volatile__("set 1f, %0; stxa %1, [%2] 0x58; flush %0; membar #Sync; jmpl %0, %%g0; nop;1:": "=&r"(pc) : "r"(c),"r"(x): "memory");})

#define set_secondary_context(c) ({unsigned long pc, x = SECONDARY_CONTEXT; __asm__ __volatile__("set 1f, %0; stxa %1, [%2] 0x58; flush %0; membar #Sync; jmpl %0, %%g0; nop;1:": "=&r"(pc) : "r"(c),"r"(x): "memory");})

#define get_tlb(s, ndx) ({unsigned long x; __asm__ __volatile__("ldxa [%1] %2, %0" : "=&r"(x) : "r"((ndx)<<3), "i"(s)); x;})

struct pmap {
};

typedef struct pmap	*pmap_t;
typedef long		pte_t;
typedef long		pde_t;

#define NPD1EPG	(PAGE_SIZE/(sizeof(pde_t)))
#define PTDPTDI	(NPD1EPG - 1)

vm_offset_t pmap_bootstrap(vm_offset_t);

extern vm_offset_t phys_avail[];

#endif /* !ASSEMBLER */
#endif /* !_MACHIINE_PMAP_H_ */
