#include <sys/types.h>
#include <sys/systm.h>
#include <machine/param.h>
#include <machine/cons.h>
#include <machine/cpu.h>
#include <machine/frame.h>
#include <machine/prom.h>
#include <machine/asi.h>
#include <machine/pmap.h>

extern unsigned long ttable_tl0, ttable_tl1, _end;
extern void setup_tba(void *);

extern void barf_trap_data(int);

static handle_t mem_node;
static struct prom_region prom_memlist[64], tmp;

#define SUN4U_PHYS_BANKS 7
vm_offset_t phys_avail[SUN4U_PHYS_BANKS*2 + 2];

void
init_sparc64()
{
	register unsigned long tick;
	extern unsigned long prom_tba;
	unsigned long *missme = (unsigned long *)(0x404000UL + 0x500000UL);
	int i, j, sz;

	cninit();
	ident_cpu();

	/* setup trap table */
	printf("prom ttable at 0x%lx\n", (prom_tba = gettba()));
	setup_tba(&ttable_tl0);
	printf("my ttable at 0x%lx\n", gettba());
	/*
	 * Prom I/O routines now start DMMU-missing but device tree
	 * routines still work!
	 */

	/* start tickin' */
	printf("pil = 0x%lx, tick = 0x%016lx, tick_cmpr = 0x%016lx\n",
	       getpil(), gettick(), gettick_cmpr());
	__asm__ __volatile__("
		rd %%tick, %0
		andn %0, %1, %0
		wrpr %0, 0, %%tick
	" : "=&r"(tick) : "r"(1UL<<63));
	settick_cmpr(142857143/100);
	printf("pil = 0x%lx, tick = 0x%016lx, tick_cmpr = 0x%016lx\n",
	       getpil(), gettick(), gettick_cmpr());

#if 0
	/* test our miss handler */
	for (i = 0; i < 5; i++) {
		printf("pointer = 0x%016lx, val = 0x%016lx\n",
		       (unsigned long)missme, *missme);
		missme += PAGE_SIZE/(sizeof(unsigned long));
	}
	printf("primary context = 0x%lx, secondary context = 0x%lx\n",
	       get_primary_context(), get_secondary_context());
	set_primary_context(0x10);
	/* access to zs registers causes a DTLB miss because it is 
	   not GLOBALLY mapped. Our DTLB miss handler makes it write to
	   pa 0x4000. Ouch ! */
	printf("can't write to zs mapped memory\n");
	/* but this trap handler can! */
	__asm__ __volatile__("ta 0x100; nop");
	set_primary_context(0);

#endif
	/* count up physical memory */
	mem_node = prom_finddevice("/memory");
	sz = (int)prom_getprop(mem_node, "available", (void *)prom_memlist,
			       sizeof(prom_memlist));
	sz /= sizeof(prom_memlist[0]);
	for (i = 0; i < sz - 1; i++)
		for (j = i + 1; j < sz; j++)
			if (prom_memlist[i].pr_start >
			    prom_memlist[j].pr_start) {
				tmp = prom_memlist[i];
				prom_memlist[i] = prom_memlist[j];
				prom_memlist[j] = tmp;
			}
	for (i = 0; i < sz; i++) {
		if (i > SUN4U_PHYS_BANKS) {
			printf("Some RAM unused, increase SUN4U_PHYS_BANKS\n");
			break;
		}
		phys_avail[i * 2] = (vm_offset_t)prom_memlist[i].pr_start;
		phys_avail[i * 2 + 1] = phys_avail[i * 2] +
		    (vm_offset_t)prom_memlist[i].pr_size;
	}
	phys_avail[i * 2] = phys_avail[i * 2 + 1] = 0;
	for (i = 0; phys_avail[i]!=0 || phys_avail[i+1]!=0; i+=2)
		printf("(0x%016lx, 0x%016lx)\n", phys_avail[i],
		       phys_avail[i+1]);
	phys_avail[0] = pmap_bootstrap(phys_avail[0]);
	barf_trap_data(5);
}

/* KCHXXX snarfed from alpha/alpha/machdep.c: copyrights ? */
void
bzero(void *buf, size_t len)
{
	caddr_t p = buf;

	while (((vm_offset_t) p & (sizeof(u_long) - 1)) && len) {
		*p++ = 0;
		len--;
	}
	while (len >= sizeof(u_long)) {
		*(u_long*) p = 0;
		p += sizeof(u_long);
		len -= sizeof(u_long);
	}
	while (len) {
		*p++ = 0;
		len--;
	}
}

void
panic(const char *str, ...)
{
	extern void back_to_prom();

	printf("page_fault\n");
	back_to_prom();
}
