#include <machine/frame.h>
#include <machine/cpu.h>
#include <machine/asi.h>
#include <machine/pmap.h>

/* this RED-states the CPU */
#define BTRAP(x)				\
	illtrap;				\
	illtrap;				\
	illtrap;				\
	illtrap;				\
	illtrap;				\
	illtrap;				\
	illtrap;				\
	illtrap;

#define SPILL(x)				\
	stx	%l0, [%sp + BIAS];		\
	stx	%l1, [%sp + BIAS + 0x08];	\
	stx	%l2, [%sp + BIAS + 0x10];	\
	stx	%l3, [%sp + BIAS + 0x18];	\
	stx	%l4, [%sp + BIAS + 0x20];	\
	stx	%l5, [%sp + BIAS + 0x28];	\
	stx	%l6, [%sp + BIAS + 0x30];	\
	stx	%l7, [%sp + BIAS + 0x38];	\
	stx	%i0, [%sp + BIAS + 0x40];	\
	stx	%i1, [%sp + BIAS + 0x48];	\
	stx	%i2, [%sp + BIAS + 0x50];	\
	stx	%i3, [%sp + BIAS + 0x58];	\
	stx	%i4, [%sp + BIAS + 0x60];	\
	stx	%i5, [%sp + BIAS + 0x68];	\
	stx	%i6, [%sp + BIAS + 0x70];	\
	stx	%i7, [%sp + BIAS + 0x78];	\
	saved;					\
	retry;					\
	nop;					\
	.align	32*4

#define FILL(x)					\
	ldx	[%sp + BIAS], %l0;		\
	ldx	[%sp + BIAS + 0x08], %l1;	\
	ldx	[%sp + BIAS + 0x10], %l2;	\
	ldx	[%sp + BIAS + 0x18], %l3;	\
	ldx	[%sp + BIAS + 0x20], %l4;	\
	ldx	[%sp + BIAS + 0x28], %l5;	\
	ldx	[%sp + BIAS + 0x30], %l6;	\
	ldx	[%sp + BIAS + 0x38], %l7;	\
	ldx	[%sp + BIAS + 0x40], %i0;	\
	ldx	[%sp + BIAS + 0x48], %i1;	\
	ldx	[%sp + BIAS + 0x50], %i2;	\
	ldx	[%sp + BIAS + 0x58], %i3;	\
	ldx	[%sp + BIAS + 0x60], %i4;	\
	ldx	[%sp + BIAS + 0x68], %i5;	\
	ldx	[%sp + BIAS + 0x70], %i6;	\
	ldx	[%sp + BIAS + 0x78], %i7;	\
	restored;				\
	retry;					\
	nop;					\
	.align	32*4

	.data
#if USE_TSB
	.globl	tsb
	.align	8*1024
tsb:
	.fill	512*2, 8, 0
#endif

	.text
	.global	setup_tba
setup_tba:
	save	%sp, -160, %sp

	mov	%i0, %o0
	call	prom_set_trap_table	! call before setting %tba
	 nop

	rdpr	%pstate, %o1
	and	%o1, ~PSTATE_IE, %o1

	wrpr	%o1, PSTATE_AG, %pstate
	setx	trap_stack, %o0, %g3
	add	%g3, -BIAS, %g3
	setx	trap_state, %o0, %g4
	setx	bad_trap, %o0, %g2

	wrpr	%o1, PSTATE_MG, %pstate
	mov	TAG_ACCESS_REG, %g2
	setx	TTE_PAMASK, %o0, %g3
	setx	trap_state, %o0, %g4
	setx	TTE_KERN, %o0, %g5
#if USE_TSB
	set	tsb, %o0
#else
	mov	1, %o0
#endif
	mov	TSB_REG, %o2
	stxa	%o0, [%o2] ASI_DMMU
	stxa	%o0, [%o2] ASI_IMMU

	wrpr	%o1, %pstate
	wrpr	%i0, %tba
	rdpr	%pstate, %o1
	or	%o1, PSTATE_IE, %o1
	wrpr	%o1, %pstate
	ret
	 restore
	sir
	.align	32

tl0_trap_enter:
	stx	%i0, [%g3 + BIAS - 0x08]
	stx	%i1, [%g3 + BIAS - 0x10]
	stx	%i2, [%g3 + BIAS - 0x18]
	stx	%i3, [%g3 + BIAS - 0x20]
	stx	%i4, [%g3 + BIAS - 0x28]
	stx	%i5, [%g3 + BIAS - 0x30]
	stx	%i6, [%g3 + BIAS - 0x38]
	stx	%i7, [%g3 + BIAS - 0x40]
	stx	%l0, [%g3 + BIAS - 0x48]
	stx	%l1, [%g3 + BIAS - 0x50]
	stx	%l2, [%g3 + BIAS - 0x58]
	stx	%l3, [%g3 + BIAS - 0x60]
	stx	%l4, [%g3 + BIAS - 0x68]
	stx	%l5, [%g3 + BIAS - 0x70]
	stx	%l6, [%g3 + BIAS - 0x78]
	stx	%l7, [%g3 + BIAS - 0x80]
	stx	%o0, [%g3 + BIAS - 0x88]
	stx	%o1, [%g3 + BIAS - 0x90]
	stx	%o2, [%g3 + BIAS - 0x98]
	stx	%o3, [%g3 + BIAS - 0xa0]
	stx	%o4, [%g3 + BIAS - 0xa8]
	stx	%o5, [%g3 + BIAS - 0xb0]
	stx	%o6, [%g3 + BIAS - 0xb8]
	stx	%o7, [%g3 + BIAS - 0xc0]

	mov	%g3, %o1
	mov	%g2, %o3
	rdpr	%pstate, %o0
	wrpr	%o0, PSTATE_AG, %pstate

	rd	%ccr, %o2
	stx	%o2, [%o1 + BIAS - 0xc8]
	stx	%g1, [%o1 + BIAS - 0xd0]
	stx	%g2, [%o1 + BIAS - 0xd8]
	stx	%g3, [%o1 + BIAS - 0xe0]
	stx	%g4, [%o1 + BIAS - 0xe8]
	stx	%g5, [%o1 + BIAS - 0xf0]
	stx	%g6, [%o1 + BIAS - 0xf8]
	stx	%g7, [%o1 + BIAS - 0x100]

	mov	%g0, %fp
	add	%o1, -0x120 - REGWIN_SZ, %sp

	call	%o3
	 nop

	add	%sp, 0x120 + REGWIN_SZ, %o1
	ldx	[%o1 + BIAS - 0xc8], %o2
	wr	%o2, 0, %ccr
	ldx	[%o1 + BIAS - 0xd0], %g1
	ldx	[%o1 + BIAS - 0xd8], %g2
	ldx	[%o1 + BIAS - 0xe0], %g3
	ldx	[%o1 + BIAS - 0xe8], %g4
	ldx	[%o1 + BIAS - 0xf0], %g5
	ldx	[%o1 + BIAS - 0xf8], %g6
	ldx	[%o1 + BIAS - 0x100], %g7
	rdpr	%pstate, %o0
	wrpr	%o0, PSTATE_AG, %pstate

	ldx	[%g3 + BIAS - 0x08], %i0
	ldx	[%g3 + BIAS - 0x10], %i1
	ldx	[%g3 + BIAS - 0x18], %i2
	ldx	[%g3 + BIAS - 0x20], %i3
	ldx	[%g3 + BIAS - 0x28], %i4
	ldx	[%g3 + BIAS - 0x30], %i5
	ldx	[%g3 + BIAS - 0x38], %i6
	ldx	[%g3 + BIAS - 0x40], %i7
	ldx	[%g3 + BIAS - 0x48], %l0
	ldx	[%g3 + BIAS - 0x50], %l1
	ldx	[%g3 + BIAS - 0x58], %l2
	ldx	[%g3 + BIAS - 0x60], %l3
	ldx	[%g3 + BIAS - 0x68], %l4
	ldx	[%g3 + BIAS - 0x70], %l5
	ldx	[%g3 + BIAS - 0x78], %l6
	ldx	[%g3 + BIAS - 0x80], %l7
	ldx	[%g3 + BIAS - 0x88], %o0
	ldx	[%g3 + BIAS - 0x90], %o1
	ldx	[%g3 + BIAS - 0x98], %o2
	ldx	[%g3 + BIAS - 0xa0], %o3
	ldx	[%g3 + BIAS - 0xa8], %o4
	ldx	[%g3 + BIAS - 0xb0], %o5
	ldx	[%g3 + BIAS - 0xb8], %o6
	ldx	[%g3 + BIAS - 0xc0], %o7
	jmpl	%g7, %g0
	 nop
	sir
	.align	32

pfault:
#if 1
	rdpr	%pstate, %g7
	wrpr	%g7, PSTATE_MG|PSTATE_AG, %pstate
	sethi	%hi(panic), %g2
	call	tl0_trap_enter
	 or	%g2, %lo(panic), %g2
#else
	stx	%g7, [%g4]
	setx	TTE_KERN | 0x4000, %g5, %g7
	stxa	%g7, [%g0] ASI_DTLB_DATA_IN_REG
	retry
#endif
	sir

	.global ttable_tl0
	.align	32*1024
ttable_tl0:
tt_resvd000:		BTRAP(0)
tt_poweron_reset:	BTRAP(0)
tt_watchdog_reset:	BTRAP(0)
tt_xir:		BTRAP(0)
tt_sir:		BTRAP(0)
tt_RED_exc:		BTRAP(0)
tt_resvd006:	BTRAP(0x6) BTRAP(0x7)
tt_ia_exc:		BTRAP(0) BTRAP(0)
tt_ia_error:		BTRAP(0)
tt_resvd00b:	BTRAP(0xb) BTRAP(0xc) BTRAP(0xd) BTRAP(0xe) BTRAP(0xf)
tt_ill_instr:		BTRAP(0)
tt_priv_opcode:		BTRAP(0)
tt_inimpl_LDD:		BTRAP(0)
tt_inimpl_STD:		BTRAP(0)
tt_resvd014:	BTRAP(0x14) BTRAP(0x15) BTRAP(0x16) BTRAP(0x17) BTRAP(0x18)
		BTRAP(0x18) BTRAP(0x18) BTRAP(0x18) BTRAP(0x19) BTRAP(0x1a)
		BTRAP(0x1b) BTRAP(0x1c) BTRAP(0x1d) BTRAP(0x1e) BTRAP(0x1f)
tt_fp_disabled:		BTRAP(0)
tt_fp_exc_ieee_754:		BTRAP(0)
tt_fp_exc_other:		BTRAP(0)
tt_tag_oflow:		BTRAP(0)
tt_cleanwin:		BTRAP(0)
tt_div_zero:		BTRAP(0)
tt_cpuerr:		BTRAP(0)
tt_resvd02a:	BTRAP(0x2a) BTRAP(0x2b) BTRAP(0x2c) BTRAP(0x2d) BTRAP(0x2e)
		BTRAP(0x2f)
tt_da_exc:		BTRAP(0) BTRAP(0)
tt_da_err:		BTRAP(0)
tt_da_prot:		BTRAP(0)
tt_misalign:		BTRAP(0)
tt_LDDF_align:		BTRAP(0)
tt_STDF_align:		BTRAP(0)
tt_priv_action:		BTRAP(0)
tt_LDQF_align:		BTRAP(0)
tt_STQF_align:		BTRAP(0)
tt_resvd03a:	BTRAP(0x3a) BTRAP(0x3b) BTRAP(0x3c) BTRAP(0x3d) BTRAP(0x3e)
		BTRAP(0x3f)
tt_async_data_err:		BTRAP(0)
tt_int01:		BTRAP(0)
tt_int02:		BTRAP(0)
tt_int03:		BTRAP(0)
tt_int04:		BTRAP(0)
tt_int05:		BTRAP(0)
tt_int06:		BTRAP(0)
tt_int07:		BTRAP(0)
tt_int08:		BTRAP(0)
tt_int09:		BTRAP(0)
tt_int0a:		BTRAP(0)
tt_int0b:		BTRAP(0)
tt_int0c:		BTRAP(0)
tt_int0d:		BTRAP(0)
tt_int0e:
	sethi	%hi(do_timer), %g2
	or	%g2, %lo(do_timer), %g2
	sethi	%hi(1f), %g7
	ba,pt	%xcc, tl0_trap_enter
	 or	%g7, %lo(1f), %g7
1:
	wr	%g0, 1, %clear_softint
	retry
	nop
tt_int0f:		BTRAP(0)
tt_resvd050:	BTRAP(0x50) BTRAP(0x51) BTRAP(0x52) BTRAP(0x53) BTRAP(0x54)
		BTRAP(0x55) BTRAP(0x56) BTRAP(0x57) BTRAP(0x58) BTRAP(0x59)
		BTRAP(0x5a) BTRAP(0x5b) BTRAP(0x5c) BTRAP(0x5d) BTRAP(0x5e)
		BTRAP(0x5f)
tt_resvd060:
	BTRAP(0x60) BTRAP(0x61) BTRAP(0x62) BTRAP(0x63)
	BTRAP(0x64) BTRAP(0x65) BTRAP(0x66) BTRAP(0x67)
tt_dtlb_miss:
	ldxa	[%g2] ASI_DMMU, %g6	! Tag Read Reg
	srlx	%g6, PD1_SHIFT - 3, %g7
	and	%g7, 0x1ff8, %g7
	ldxa	[%g1+%g7] ASI_PHYS_USE_EC, %g7
	brgez	%g7, pfault
	 and	%g7, %g3, %g5		! got PD2

	srlx	%g6, PD2_SHIFT - 3, %g7
	and	%g7, 0x1ff8, %g7
	ldxa	[%g5+%g7] ASI_PHYS_USE_EC, %g7
	brgez	%g7, pfault
	 and	%g7, %g3, %g5		! got PT

	ldxa	[%g0] ASI_DMMU_TSB_8KB_PTR_REG, %g7
	srlx	%g7, 1, %g7
	ldxa	[%g5+%g7] ASI_PHYS_USE_EC, %g7
	brgez	%g7, pfault
	 nop
	stxa	%g7, [%g0] ASI_DTLB_DATA_IN_REG
	retry

	.align	32*4
foobar:		BTRAP(0x6c) BTRAP(0x6d) BTRAP(0x6e) BTRAP(0x6f)
tt_impldep_exc:	BTRAP(0x70) BTRAP(0x71) BTRAP(0x72) BTRAP(0x73) BTRAP(0x74)
		BTRAP(0x75) BTRAP(0x76) BTRAP(0x77) BTRAP(0x78) BTRAP(0x79)
		BTRAP(0x7a) BTRAP(0x7b) BTRAP(0x7c) BTRAP(0x7d) BTRAP(0x7e)
		BTRAP(0x7f)
tt_spill_norm0:	SPILL(0)
tt_spill_norm1:	SPILL(0)
tt_spill_norm2:	SPILL(0)
tt_spill_norm3: SPILL(0)
tt_spill_norm4:	SPILL(0)
tt_spill_norm5:	SPILL(0)
tt_spill_norm6:	SPILL(0)
tt_spill_norm7:	SPILL(0)
tt_spill_other0: SPILL(0)
tt_spill_other1: SPILL(0)
tt_spill_other2: SPILL(0)
tt_spill_other3: SPILL(0)
tt_spill_other4: SPILL(0)
tt_spill_other5: SPILL(0)
tt_spill_other6: SPILL(0)
tt_spill_other7: SPILL(0)
tt_fill_norm0:	FILL(0)
tt_fill_norm1:	FILL(0)
tt_fill_norm2:	FILL(0)
tt_fill_norm3:	FILL(0)
tt_fill_norm4:	FILL(0)
tt_fill_norm5:	FILL(0)
tt_fill_norm6:	FILL(0)
tt_fill_norm7:	FILL(0)
tt_fill_other0:	FILL(0)
tt_fill_other1:	FILL(0)
tt_fill_other2:	FILL(0)
tt_fill_other3:	FILL(0)
tt_fill_other4:	FILL(0)
tt_fill_other5:	FILL(0)
tt_fill_other6:	FILL(0)
tt_fill_other7:	FILL(0)
ttable_resvd100:
	sethi	%hi(bad_trap), %g2
	or	%g2, %lo(bad_trap), %g2
	sethi	%hi(1f), %g7
	ba,pt	%xcc, tl0_trap_enter
	or	%g7, %lo(1f), %g7
1:
	done
	nop; nop;
tt_resvd101:
	.fill	32*(512-0x101), 1, 0

	.global ttable_tl1
ttable_tl1:
tl1_tt_resvd000:		BTRAP(0)
tl1_tt_poweron_reset:	BTRAP(0)
tl1_tt_watchdog_reset:	BTRAP(0)
tl1_tt_xir:		BTRAP(0)
tl1_tt_sir:		BTRAP(0)
tl1_tt_RED_exc:		BTRAP(0)
tl1_tt_resvd006:	BTRAP(0x6) BTRAP(0X7)
tl1_tt_ia_exc:		BTRAP(0) BTRAP(0)
tl1_tt_ia_error:	BTRAP(0)
tl1_tt_resvd00b:	BTRAP(0xb) BTRAP(0xc) BTRAP(0xd) BTRAP(0xe) BTRAP(0xf)
tl1_tt_ill_instr:		BTRAP(0)
tl1_tt_priv_opcode:		BTRAP(0)
tl1_tt_inimpl_LDD:		BTRAP(0)
tl1_tt_inimpl_STD:		BTRAP(0)
tl1_tt_resvd014:	BTRAP(0x14) BTRAP(0x15) BTRAP(0x16) BTRAP(0x17) BTRAP(0x18)
		BTRAP(0x18) BTRAP(0x18) BTRAP(0x18) BTRAP(0x19) BTRAP(0x1a)
		BTRAP(0x1b) BTRAP(0x1c) BTRAP(0x1d) BTRAP(0x1e) BTRAP(0x1f)
tl1_tt_fp_disabled:		BTRAP(0)
tl1_tt_fp_exc_ieee_754:		BTRAP(0)
tl1_tt_fp_exc_other:		BTRAP(0)
tl1_tt_tag_oflow:		BTRAP(0)
tl1_tt_cleanwin:		BTRAP(0)
tl1_tt_div_zero:		BTRAP(0)
tl1_tt_cpuerr:		BTRAP(0)
tl1_tt_resvd02a:	BTRAP(0x2a) BTRAP(0x2b) BTRAP(0x2c) BTRAP(0x2d) BTRAP(0x2e)
		BTRAP(0x2f)
tl1_tt_da_exc:		BTRAP(0) BTRAP(0)
tl1_tt_da_err:		BTRAP(0)
tl1_tt_da_prot:		BTRAP(0)
tl1_tt_misalign:		BTRAP(0)
tl1_tt_LDDF_align:		BTRAP(0)
tl1_tt_STDF_align:		BTRAP(0)
tl1_tt_priv_action:		BTRAP(0)
tl1_tt_LDQF_align:		BTRAP(0)
tl1_tt_STQF_align:		BTRAP(0)
tl1_tt_resvd03a:	BTRAP(0x3a) BTRAP(0x3b) BTRAP(0x3c) BTRAP(0x3d) BTRAP(0x3e)
		BTRAP(0x3f)
tl1_tt_async_data_err:		BTRAP(0)
tl1_tt_int01:		BTRAP(0)
tl1_tt_int02:		BTRAP(0)
tl1_tt_int03:		BTRAP(0)
tl1_tt_int04:		BTRAP(0)
tl1_tt_int05:		BTRAP(0)
tl1_tt_int06:		BTRAP(0)
tl1_tt_int07:		BTRAP(0)
tl1_tt_int08:		BTRAP(0)
tl1_tt_int09:		BTRAP(0)
tl1_tt_int0a:		BTRAP(0)
tl1_tt_int0b:		BTRAP(0)
tl1_tt_int0c:		BTRAP(0)
tl1_tt_int0d:		BTRAP(0)
tl1_tt_int0e:		BTRAP(0)
tl1_tt_int0f:		BTRAP(0)
tl1_tt_resvd050:
	BTRAP(0x50) BTRAP(0x51) BTRAP(0x52) BTRAP(0x53) BTRAP(0x54) BTRAP(0x55)
	BTRAP(0x56) BTRAP(0x57) BTRAP(0x58) BTRAP(0x59) BTRAP(0x5a) BTRAP(0x5b)
	BTRAP(0x5c) BTRAP(0x5d) BTRAP(0x5e) BTRAP(0x5f)
tl1_tt_resvd060:
	BTRAP(0x60) BTRAP(0x61) BTRAP(0x62) BTRAP(0x63)
	BTRAP(0x64) BTRAP(0x65) BTRAP(0x66) BTRAP(0x67)
tl1_tt_dtlb_miss:
	.fill	32, 4, 0
	BTRAP(0x6c) BTRAP(0x6d) BTRAP(0x6e) BTRAP(0x6f)
tl1_tt_impldep_exc:
	BTRAP(0x70) BTRAP(0x71) BTRAP(0x72) BTRAP(0x73)
	BTRAP(0x74) BTRAP(0x75) BTRAP(0x76) BTRAP(0x77)
	BTRAP(0x78) BTRAP(0x79) BTRAP(0x7a) BTRAP(0x7b)
	BTRAP(0x7c) BTRAP(0x7d) BTRAP(0x7e) BTRAP(0x7f)
tl1_tt_spill_norm0:	SPILL(0)
tl1_tt_spill_norm1:	SPILL(0)
tl1_tt_spill_norm2:	SPILL(0)
tl1_tt_spill_norm3: SPILL(0)
tl1_tt_spill_norm4:	SPILL(0)
tl1_tt_spill_norm5:	SPILL(0)
tl1_tt_spill_norm6:	SPILL(0)
tl1_tt_spill_norm7:	SPILL(0)
tl1_tt_spill_other0: SPILL(0)
tl1_tt_spill_other1: SPILL(0)
tl1_tt_spill_other2: SPILL(0)
tl1_tt_spill_other3: SPILL(0)
tl1_tt_spill_other4: SPILL(0)
tl1_tt_spill_other5: SPILL(0)
tl1_tt_spill_other6: SPILL(0)
tl1_tt_spill_other7: SPILL(0)
tl1_tt_fill_norm0:	FILL(0)
tl1_tt_fill_norm1:	FILL(0)
tl1_tt_fill_norm2:	FILL(0)
tl1_tt_fill_norm3:	FILL(0)
tl1_tt_fill_norm4:	FILL(0)
tl1_tt_fill_norm5:	FILL(0)
tl1_tt_fill_norm6:	FILL(0)
tl1_tt_fill_norm7:	FILL(0)
tl1_tt_fill_other0:	FILL(0)
tl1_tt_fill_other1:	FILL(0)
tl1_tt_fill_other2:	FILL(0)
tl1_tt_fill_other3:	FILL(0)
tl1_tt_fill_other4:	FILL(0)
tl1_tt_fill_other5:	FILL(0)
tl1_tt_fill_other6:	FILL(0)
tl1_tt_fill_other7:	FILL(0)
tl1_ttable_resvd100:	BTRAP(0)
tl1_ttable_resvd101:
	.fill	32*(512-0x101), 1, 0
end_ttable:
	.xword	0

