/*-
 * Copyright 1979, 1980, 1983, 1986, 1988, 1989, 1991, 1992, 1993, 1994
 * 	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by the University of
 * California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

/* zs8530 chip `driver': 9600,8,n,1,- mode only */

#include <sys/param.h>
#include <sys/systm.h>
#include <machine/prom.h>
#include <machine/zs.h>

static char zsname[] = "/sbus/zs@f,1100000";
static handle_t zsnode = -1UL;
/* XXX must be made static */
struct zschip *zs = 0;

unsigned long zs_debug = 1;

void
zscnprobe(struct consdev *cp)
{
	unsigned long retval;

	if ((zsnode = prom_finddevice(zsname)) == -1UL) {
		prom_printf("zs0: Can't find %s\n", zsname);
		return;
	}
	prom_printf("zs0: zsnode = 0x%0lx\n", zsnode);
	if ((zs = (struct zschip *)prom_getpropaddr(zsnode, "address")) ==
	    0) {
		prom_printf("zs0: Cant get address prop (0x%lx)\n", retval);
		return;
	}

	prom_printf("zs0: mapped at = 0x%lx\n", (unsigned long)zs);

	cp->cn_pri = CN_REMOTE;
}

void
zscninit(struct consdev *cp)
{
	unsigned int brg;
	volatile unsigned int dummy;

	prom_printf("zs0: zscninit() 0x%lx\n", &zs->zs_chan[ZS_CHAN_A]);
#if defined(PROM_CONSOLE_IS_NOT_ZS)
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_HWRST);
	dummy = ZS_READ(&zs->zs_chan[ZS_CHAN_A], ZS_R0);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R4, ZS4_EVENP|ZS4_CLK_X16|ZS4_SB1);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R3, ZS3_RX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R5, ZS5_TX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_NOVEC);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R10, ZS10_NRZ);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R11, ZS11_RX_BR|ZS11_TX_BR);
	brg = BPS_TO_BRG(ZS_INCLK/16, 9600);
	prom_printf("\tbrg = 0x%x\n", brg);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R12, brg & 0xff);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R13, (brg>>8) & 0xff);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R14, ZS14_BRSRC_PCLK);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R14, ZS14_BRSRC_PCLK|ZS14_BREN);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R3, ZS3_RX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R5, ZS5_TX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R15, ZS15_DCD_IE);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_MASTER_IE|ZS9_NOVEC);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R0, ZS0_RESET_EXTINT);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R0, ZS0_RESET_EXTINT);

	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R3, 0);
	zs->zs_chan[ZS_CHAN_A].zc_ctrl = ZS0_RESET_EXTINT;
	zs->zs_chan[ZS_CHAN_A].zc_ctrl = ZS0_RESET_ERRORS;
	dummy = zs->zs_chan[ZS_CHAN_A].zc_data;
	dummy = zs->zs_chan[ZS_CHAN_A].zc_data;
	dummy = zs->zs_chan[ZS_CHAN_A].zc_data;
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_RESET_A);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R4, ZS4_EVENP|ZS4_CLK_X16|ZS4_SB1);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R3, ZS3_RX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R5, ZS5_DTR|ZS5_TX8);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_NOVEC);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R10, ZS10_NRZ);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R11, ZS11_RX_BR|ZS11_TX_BR);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R12, brg & 0xff);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R13, (brg>>8) & 0xff);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R14, ZS14_BRSRC_PCLK);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R14, ZS14_BRSRC_PCLK|ZS14_BREN);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R3, ZS3_RX8|ZS3_RXEN);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R5, ZS5_DTR|ZS5_TX8|ZS5_TXEN);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R15,
		 ZS15_DCD_IE|ZS15_CTS_IE|ZS15_TXU_IE|ZS15_BRK_IE);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R0, ZS0_RESET_EXTINT);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R0, ZS0_RESET_ERRORS);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R1, ZS1_EXTINT_EN|ZS1_INTALL_RX);
	ZS_WRITE(&zs->zs_chan[ZS_CHAN_A], ZS_R9, ZS9_MASTER_IE);
#endif
}

void
zscnputc(dev_t d, int c)
{
	int loops = 2000;

	while ((zs->zs_chan[ZS_CHAN_A].zc_ctrl & ZS0_TX_RDY) == 0 && loops--)
		DELAY(5);

	zs->zs_chan[ZS_CHAN_A].zc_data = (char)c;
	loops = 2000;
	while (!(ZS_READ(&zs->zs_chan[ZS_CHAN_A], ZS_R1) & ZS1_ALL_SENT) && loops--)
		DELAY(5);
}
